<?php

/**
 * @file
 * Menu builder functions for Node Compare.
 */

/**
 * Form builder function for module settings.
 */
function node_compare_admin_settings($form, &$form_state) {
  $form['main'] = array(
    '#type' => 'fieldset',
  );
  $form['main']['node_compare_show_history'] = array(
    '#type' => 'checkbox',
    '#title' => t('Show recent comparisons'),
    '#description' => t('Display the recent comparisons list in the comparison block.'),
    '#default_value' => variable_get('node_compare_show_history', FALSE),
  );

  if ($types = node_type_get_names()) {
    $form['fields'] = array(
      '#type' => 'fieldset',
      '#title' => t('Types and fields'),
      '#description' => t("Choose separately for each type of node which fields are allowed to compare.<br />Types of nodes for which you do not select any field, are excluded from the comparison."),
      '#collapsible' => TRUE,
    );

    foreach ($types as $type => $label) {
      $fields = field_info_instances('node', $type);
      if ($fields) {
        $form['fields'][$type] = array(
          '#type' => 'fieldset',
          '#title' => t('Type: @type', array('@type' => $label)),
          '#collapsible' => TRUE,
          '#collapsed' => TRUE,
        );
        $field_list = array();
        foreach ($fields as $field => $data) {
          $field_list[$field] = $data['label'] . ' (' . $field . ')';
        }
        $form['fields'][$type]['node_compare_type_' . $type] = array(
          '#type' => 'checkboxes',
          '#options' => $field_list,
          '#default_value' => variable_get('node_compare_type_' . $type, array()),
        );
      }
    }
  }

  $form['array_filter'] = array(
    '#type' => 'value',
    '#value' => TRUE,
  );
  return system_settings_form($form);
}

/**
 * Add/remove nodes to compare.
 */
function node_compare_toggle($node, $mode = '') {
  node_compare_sess_update($node->type, $node->nid, $node->title);
  // Checks ajax mode.
  if ($mode == 'ajax') {
    $commands[] = ajax_command_prepend('body', '<div class="compare-messages-ajax">' . theme('status_messages') . '</div>');
    $commands[] = ajax_command_html('#node-compare-items', theme('node_compare_block_content'));
    $commands[] = ajax_command_replace('#compare-toggle-' . $node->nid, theme('node_compare_toggle_link', array('nid' => $node->nid)));
    $page = array('#type' => 'ajax', '#commands' => $commands);
    ajax_deliver($page);
  }
  // If JS disabled, then redirect the user back to the page where user came from.
  else {
    drupal_goto();
  }
}

/**
 * Generates a page with a comparative table.
 */
function node_compare_page() {
  $nids = func_get_args();
  $type = array_shift($nids);

  if (isset($type, $nids) && count($nids) > 1) {
    $header = array();
    foreach ($nodes = node_load_multiple($nids, array(), TRUE) as $node) {
      if (node_access('view', $node) && $node->type == $type) {
        $link_options = array('attributes' => array(
            'title' => $node->title,
            'class' => array('compare-item'),
          ),
        );
        $header[$node->nid] = l($node->title, 'node/' . $node->nid, $link_options);
      }
    }
    if (count($header) == count($nids)) {
      $fields = variable_get('node_compare_type_' . $type, array());
      $rows = array();

      foreach ($fields as $field_name) {
        $instance = field_info_instance('node', $field_name, $type);
        $row = array(array('data' => $instance['label'], 'class' => 'compare-field-label'));
        foreach (array_keys($header) as $nid) {
          $field = field_view_field('node', $nodes[$nid], $field_name, array('label' => 'hidden'));
          $row[] = $field ? render($field) : t('N/A');
        }
        $rows[] = $row;
      }
      array_unshift($header, t('Properties:'));
      if ($rows) {
        $output = array(
          '#theme' => 'table',
          '#header' => $header,
          '#rows' => $rows,
          '#attributes' => array(
            'class' => array('comparison-table'),
          ),
        );
        if (user_is_logged_in() || !variable_get('cache', FALSE)) {
          $output['#cache'] = array(
            'keys' => array('node_compare', $type, implode('/', $nids)),
            'bin' => 'cache_page',
            'granularity' => DRUPAL_CACHE_PER_ROLE,
          );
        }
        return $output;
      }
    }
  }
  return MENU_NOT_FOUND;
}

/**
 * Update session when handling the nodes selected for comparison.
 */
function node_compare_sess_update($type, $nid, $title) {
  if (isset($_SESSION['node_compare']) && $_SESSION['node_compare']['type'] == $type) {
    if (isset($_SESSION['node_compare']['nids'][$nid])) {
      unset($_SESSION['node_compare']['nids'][$nid]);
      drupal_set_message(t('%title has been removed from the comparison.', array('%title' => $title)));
      return;
    }
  }
  else {
    $_SESSION['node_compare']['type'] = $type;
    $_SESSION['node_compare']['nids'] = array();
  }
  $_SESSION['node_compare']['nids'][$nid] = $title;
  drupal_set_message(t('%title has been added for comparison.', array('%title' => $title)));
}

/**
 * Processing for nodes selected for comparison by the current user.
 */
function node_compare_me() {
  if (isset($_SESSION['node_compare']['type'], $_SESSION['node_compare']['nids'])) {
    $sess = $_SESSION['node_compare'];
    unset($_SESSION['node_compare']);
    $nids = array_keys($sess['nids']);
    $nids = '/' . implode('/', $nids);
    $url = 'compare/type/' . $sess['type'] . $nids;
    if (variable_get('node_compare_show_history', FALSE)) {
      $_SESSION['node_compare_history'][time()] = $url;
    }
    drupal_goto($url);
  }
}

